/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <ctime>
#include <ostream>
#include <sstream>
#include <string>

#include <Foundation/logging/LogLevel.hpp>
#include <Foundation/osthreads/MutexGuard.hpp>
#include <Foundation/logging/LogManager.hpp>

#include "logging/LogManagerDefaultDelegate.hpp"

// Anonymous namespace
namespace
{


    // TODO: This should be moved to OsSupport
    std::string basename(const std::string & f) 
    {
        const char DELIM='/';
        std::string::size_type i= f.rfind(DELIM);
        if (i != std::string::npos) {
            std::string res = f.substr(i+1,f.size());
            return res;
        } else
            return f;
    }
    
} // End of anonymous namespace


#if defined CDMW_LOG_USE_BASENAME
// Force call to basename() for filenames. This will have a performance impact!
#define CDMW_LOG_BASENAME(X) basename(X)
#else
#define CDMW_LOG_BASENAME(X) X
#endif

namespace Cdmw
{
    namespace Logging
    {
        /**
         * Constructor.
         */
        LogManagerDefaultDelegate::LogManagerDefaultDelegate()
            throw(OutOfMemoryException)
            :
            m_ofstream(),
            m_postream(&std::cout)
        {
        }

        /**
         * Constructor.
         */
        LogManagerDefaultDelegate::LogManagerDefaultDelegate(std::string file_name)
            throw(OutOfMemoryException,
                  BadParameterException)
            :
            m_ofstream(),
            m_postream(&std::cout)
        {
            m_ofstream.open(file_name.c_str(), std::ios::out);

            if (! m_ofstream)
            {
                throw Cdmw::BadParameterException(
                            std::string("Couldn't create output file (")
                            + file_name + ") for logger!");
            }
            else 
            {
                m_postream = &m_ofstream;
                std::cout << "CDMW Log file set to " << file_name
                          << std::endl;
            }
        }


        /**
         * Destructor.
         */
        LogManagerDefaultDelegate::~LogManagerDefaultDelegate()
            throw()
        {
            try 
            {
                m_postream = &std::cout;
                m_ofstream.close();
            }
            catch (...)
            {
                // Ignore ...
            }
        }


        /**
         * Converts a LogHeader into a formatted string.
         */
        void LogManagerDefaultDelegate::log_header_to_string(std::ostringstream& out_stream,
                                                             const CdmwLogging::LogHeader& header)
        {
    
            time_t ltime = header.time_stamping.seconds;
            char buffer1[100], buffer2[100];

            // use time structure to build a customized time string.
            tm now;
            ::localtime_r(&ltime, &now);

            // write time with second precision in buffer
            strftime(buffer1, 100, "%Y/%m/%d %H:%M:%S", &now);
            // add milliseconds and microseconds in buffer
            sprintf(buffer2, 
                    "%.3u:%.3u ",
                    (header.time_stamping.microseconds) / 1000,
                    (header.time_stamping.microseconds) % 1000 
                    );

            (out_stream) << buffer1 << ":" << buffer2
                          << " " << LevelUtilities::Level_To_String(header.level)
                          << " [" << header.component_name << "]"
                          << " " << header.user_domain 
                          << " [tid=" << header.thread_id << "]"
                          << " (" << CDMW_LOG_BASENAME(header.file_name)
                          << ":" << header.file_line << ")";
        }

        /**
         * Send a message to the logger.
         */
        bool
        LogManagerDefaultDelegate::add_log(const CdmwLogging::LogContext& context,
                                           const CdmwLogging::LogHeader& header,
                                           const char* msg)
            throw(InternalErrorException)
        {
            // context not logged
            std::ostringstream buffer;
            this->log_header_to_string(buffer, header);
            buffer << " [" << context.ndc.in() << "]" << "\t" << msg;

            CDMW_MUTEX_GUARD(m_mutex);
            (*m_postream) << buffer.str() << std::endl;
            return true;
        }

    } // end namespace Logging
} // end namespace Cdmw
