/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */
package cdmw.logging;

import static cdmw.common.Assert.assertionFailed;

import java.io.BufferedWriter;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import cdmw.common.BadParameterException;
import cdmw.common.NotFoundException;

import com.thalesgroup.CdmwLogging.LogCollector;
import com.thalesgroup.CdmwLogging.LogHeader;
import com.thalesgroup.CdmwLogging.LogContext;
import com.thalesgroup.CdmwLogging.StructuredLog;

public class LogManager
{
    private static LogManager M_Instance;

    private LogManagerDelegate m_defaultDelegate;
    private LogManagerDelegate m_alternateDelegate;
    private FatalHandler m_fatalHandler;
    private FilterMngr m_filterManager;
    private LevelType m_level;
    private boolean m_isVerbose;

    private String m_applicationName;
    private String m_processName;

    /**
     * Initialise the LogManager.
     *
     * It creates a default delegate, a target to the logs.
     *
     * @return the LogManager that has been initialised.
     */

    protected LogManager() {
        m_defaultDelegate = null;
        m_alternateDelegate = null;
        m_fatalHandler = null;
        m_filterManager = null;
        m_level = null;
        m_isVerbose = false;
        m_applicationName = "";
        m_processName = "";
    }

    /**
     * Initialise the LogManager.
     *
     * It creates a default delegate, a target to the logs.
     * Command-line arguments will be used to initialise the
     * default logger.
     *
     * @param args arguments defined in command line   
     * 
     * @return the LogManager that has been initialised.
     */
    protected LogManager(String[] args) {
        m_defaultDelegate = null;
        m_alternateDelegate = null;
        m_fatalHandler = null;
        m_level = new LevelType();
        m_isVerbose = false;
        m_applicationName = "";
        m_processName = "";

        // create the filter manager with default levels
        m_level.setType(LogLevel.DEFAULT_LEVEL);
        m_filterManager = new FilterMngr(m_level.getType());

        if (args.length != 0) {
            // parse log level option
            //
            try {
                LevelType level = this.readLevel(args);

                // set the new level if parsing succeded
                m_filterManager.setDefaultLevel(level);
            }
            catch (NotFoundException nfe) {
                // no level indicated, default is convenient
            }
            catch (BadParameterException bde) {
                System.err.println("Invalid option will be ignored: " + bde.what());
            }

            // parse log file option
            //
            String fileName = null;
            try {
                fileName = this.readOutputFile(args);
                m_defaultDelegate = new LogManagerDefaultDelegate(fileName);    
            }
            catch (NotFoundException nfe) {
                // not activated
            }
            catch (BadParameterException bde) {
                System.err.println("Invalid option will be ignored: " + bde.what());
            }

            // parse verbose mode
            //
            try {
                m_isVerbose = this.readVerboseMode(args);
            }
            catch (BadParameterException bde) {
                System.err.println("Invalid option will be ignored: " + bde.what());
                // let to default
            }
        }

        if (m_defaultDelegate == null) {
            m_defaultDelegate = new LogManagerDefaultDelegate();
        }
    }

    /**
     * Initialise the LogManager.
     */
    public static synchronized LogManager Init(String[] args) {
        if (M_Instance == null)
        { 
            M_Instance = new LogManager(args);
        }

        return M_Instance;
    }

    /**
     * Send a message to the logger.
     *
     * @param header the message header.
     * @param msg the message.
     *
     * @return true if successful.
     */
    public boolean addLog(LogHeader header, String msg) {
        return this.addLog(header, msg, "");
    }

    /**
     * Send a message to the logger.
     *
     * @param header the message header.
     * @param msg the message.
     * @param ndc the Nested Diagnostic Context.
     *
     * @return true if successful.
     */
    public boolean addLog(LogHeader header, String msg, String ndc) {
        boolean result = true;
        LogContext context = this.createCurrentContext();
        context.ndc = ndc;

        if (m_isVerbose || m_alternateDelegate == null) {
            result &= m_defaultDelegate.addLog(context, header, msg);
        }
        if (m_alternateDelegate != null) {
            result &= m_alternateDelegate.addLog(context, header, msg);
        }

        // if it's a FATAL message and a FatalHandler is registered:
        // call it
        if ((header.level == LogLevel.FATAL) && m_fatalHandler != null) 
        {
            m_fatalHandler.onFatalLog(header, msg);
        }
        
        return result;
    }

    /**
     * Register an alternate delegate.
     *
     * @param logmgr the alternate delegate.
     *
     * @return the previous registered alternate delegate.
     */

    public synchronized LogManagerDelegate registerAlternateDelegate(LogManagerDelegate alt) {
        LogManagerDelegate old = m_alternateDelegate;
        m_alternateDelegate = alt;

        return old;
    } 

    /**
     * Unregister the alternate delegate.
     *
     * @param logmgr the alternate delegate.
     *
     * @return true if the alternate delegate was previously registered.
     */

    public synchronized boolean unregisterAlternateDelegate(LogManagerDelegate alt) {

        boolean result = false;

        if (alt == m_alternateDelegate) 
        {
            result = true;
            m_alternateDelegate = null;
        }

        return result;
     }

    /**
     * Register the FatalHandler.
     *
     * @param handler the FatalHandler.
     *
     * @return the previous registered FatalHandler.
     */

    public synchronized FatalHandler registerFatalHandler(FatalHandler handler) {
        FatalHandler old = m_fatalHandler;
        m_fatalHandler = handler;

        return old;
    } 

    /**
     * Unregister the FatalHandler.
     *
     * @param handler the FatalHandler.
     *
     * @return true if the FatalHandler was previously registered.
     */

    public synchronized boolean unregisterFatalHandler(FatalHandler handler) {

        boolean result = false;

        if (handler == m_fatalHandler) 
        {
            result = true;
            m_fatalHandler = null;
        }

        return result;
     }

    /**
     * Get the LogManager.
     *
     * @return the LogManager Instance.
     */

    public static LogManager Instance() {
        if (M_Instance == null) {
            assertionFailed();
        }
        return M_Instance;
     }

    /**
     * Get the alternate delegate.
     *
     * @return the alternate delegate.
     */
    public LogManagerDelegate getAlternateDelegate() {
        return m_alternateDelegate;
    }

    /**
     * Get the filter manager.
     *
     * @return the filter manager.
     */
    public FilterMngr getFilterManager() {
        return m_filterManager;
    }

    /**
     * Set the application name.
     * @param applicationName application name
     */
    public void setApplicationName(String applicationName) {
        m_applicationName = applicationName;
    }

    /**
     * Get the application name.
     * @return the application name
     */
    public String getApplicationName() {
        return m_applicationName;
    }

    /**
     * Set the process name.
     * @param processName process name
     */
    public void setProcessName(String processName) {
        m_processName = processName;
    }

    /**
     * Get the process name.
     * @return the process name
     */
    public String getprocessName() {
        return m_processName;
    }

    /**
     * Create the LogContext of the calling thread.
     * @return the built and filled context object
     */
    protected LogContext createCurrentContext() {
        LogContext context = new LogContext();
        context.host_name = cdmw.ossupport.OS.getHostname();
        context.application_name = m_applicationName;
        context.process_name = m_processName;
        context.process_id = 0;
        context.ndc = "";
        return context;
    }

    /**
     * Retrieve the log level value described in command line arguments.
     * 
     * @param args input arguments
     * @return the level found
     * @throws NotFoundException
     * @throws BadParameterException
     */
    protected LevelType readLevel(String args[])
        throws NotFoundException, BadParameterException {

        LevelType level = null;
        String levelOption = 
            cdmw.common.Options.getOptionValue(args, cdmw.common.Options.LOG_LEVEL_OPTION);

         if (levelOption == "no") {
             throw new NotFoundException();
         }
         else if (levelOption == "yes") {
             throw new BadParameterException(cdmw.common.Options.LOG_LEVEL_OPTION, "");
         }
         else {
             try {
                 level = new LevelType(this.parseLogLevelOption(levelOption));
             }
             catch (InvalidAlgorithmParameterException iape) {
                 throw new BadParameterException(cdmw.common.Options.LOG_LEVEL_OPTION, levelOption);
             }
         }

         return level;
    }

    /**
     * Retrieve the output log file described in command line arguments.
     * 
     * @param args
     * @return the found file name
     * @throws NotFoundException
     * @throws BadParameterException
     */
    protected String readOutputFile(String args[])
        throws NotFoundException, BadParameterException {

        String fileOption = 
            cdmw.common.Options.getOptionValue(args, cdmw.common.Options.LOG_FILE_OPTION);

        if (fileOption == "no") {
            throw new NotFoundException();
        }
        else if (fileOption == "yes") {
            throw new BadParameterException(cdmw.common.Options.LOG_FILE_OPTION, "");
        }
        
        return fileOption;
    }

    /**
     * Retrieve if the verbose mode is activated in command line arguments.
     * 
     * @param args input arguments
     * @return yes if found and activated, false otherwise.
     * @throws BadParameterException
     */
    protected boolean readVerboseMode(String args[])
        throws BadParameterException {

        boolean verbose;
        String levelOption = 
            cdmw.common.Options.getOptionValue(args, cdmw.common.Options.LOG_VERBOSE_OPTION);

         if (levelOption == "no") {
             verbose = false;
         }
         else if (levelOption == "yes") {
             verbose = true;
         }
         else {
             throw new BadParameterException(cdmw.common.Options.LOG_VERBOSE_OPTION, levelOption);
         }

         return verbose;
    }

    /**
     * Converts a command line option into the corresponding log
     * level.
     * The level must be defined accordingly to the next grammar:
     *     Integer   |   (Integer [+-])? String ([+-] String )*
     *
     * It means next rules must be respected:
     * - either a number (corresponding to a level mask)
     * - or a string with level names (or mask names) separated
     *   with + or - (for activation or deactivation)
     * - or a number followed by a + or - and level names.
     *
     * @param levelOption the user option read in the command_line
     * that must be respectful with the above grammar.
     *
     * @return the log level resulting from the combination of
     * all level names with associated operations.
     *
     * @exception BadParameterException raised when the level 
     * description is not valid
     */
    protected long parseLogLevelOption(String levelOption)
        throws InvalidAlgorithmParameterException
    {
        long level = 0;
        char op = '+'; // the first level is always added!
        int radix = 10;

        // possibly removes the hexa marker
        String parsedOpt = levelOption;
        if (levelOption.startsWith("0x"))
        {
            parsedOpt = levelOption.substring(2);
            radix = 16;
        }
        StringTokenizer strLevel = new StringTokenizer(parsedOpt, "+-", true);

        // first considers the case when the stream starts with 
        // a numeric level, hexadecimal or decimal
        //
        try {
            level = Long.parseLong(strLevel.nextToken(), radix);

            // read the potential operator
            op = strLevel.nextToken().charAt(0);
        }
        catch (NumberFormatException nfe)
        {
            // let the level at 0 and initializes the stream
            strLevel = new StringTokenizer(levelOption, "+-", true);
        }
        catch (NoSuchElementException nsee)
        {
            // it means end of string reached
        }

        // then adds or cuts a new level to each lap
        //
        while (strLevel.hasMoreTokens())
        {
            long new_level = this.read_level(strLevel.nextToken());
            switch (op)
            {
            case '+': // A OR B
                level |= new_level;
                break;
            case '-': // (A XOR B) AND (NOT B)
                level = (level ^ new_level) & (~new_level);
                break;
            default:
                throw new
                    InvalidAlgorithmParameterException();
            }

            // read the potential operator
            try {
                op = strLevel.nextToken().charAt(0);
            }
            catch (NoSuchElementException ioobe) { }
        }

        return level;
    }

    /**
     * Reads in the stream a log level that is written in a string.
     * Supports all log levels enumerated in LogLevel.hpp, 
     * and also: DEFAULT, ALL, NONE
     *
     * Note that the algorithm is not efficient, the comparison is 
     * done for each existing level until the good one is found. As
     * this is used in initialization, it is considered as 
     * satisfactory.
     *
     * @param strLevel the string representation of a level.
     *
     * @return the numerous level corresponding to the read level
     * name.
     * 
     * @exception BadParameterException when an unknown identifier
     * is discovered into the stream
     */
    protected long read_level(final String strLevel)
        throws InvalidAlgorithmParameterException
    {
        long level = 0;

        if (strLevel.equals(LogLevel.sALL))
        {
            level = cdmw.logging.LogLevel.ALL_LEVELS_SET;
        }
        else if (strLevel.equals(LogLevel.sDEFAULT))
        {
            level = cdmw.logging.LogLevel.DEFAULT_LEVEL;
        }
        else if (strLevel.equals(LogLevel.sNONE))
        {
            level = cdmw.logging.LogLevel.NONE_LEVEL;
        }

        else if (strLevel.equals(LogLevel.sFATAL))
        {
            level = cdmw.logging.LogLevel.FATAL;
        }
        else if (strLevel.equals(LogLevel.sERROR))
        {
            level = cdmw.logging.LogLevel.ERROR;
        }
        else if (strLevel.equals(LogLevel.sWARN))
        {
            level = cdmw.logging.LogLevel.WARN;
        }
        else if (strLevel.equals(LogLevel.sINFO))
        {
            level = cdmw.logging.LogLevel.INFO;
        }

        else if (strLevel.equals(LogLevel.sDEBUG_1))
        {
            level = cdmw.logging.LogLevel.DEBUG_1;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_2))
        {
            level = cdmw.logging.LogLevel.DEBUG_2;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_3))
        {
            level = cdmw.logging.LogLevel.DEBUG_3;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_4))
        {
            level = cdmw.logging.LogLevel.DEBUG_4;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_5))
        {
            level = cdmw.logging.LogLevel.DEBUG_5;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_6))
        {
            level = cdmw.logging.LogLevel.DEBUG_6;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_7))
        {
            level = cdmw.logging.LogLevel.DEBUG_7;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_8))
        {
            level = cdmw.logging.LogLevel.DEBUG_8;
        }
        else if (strLevel.equals(LogLevel.sDEBUG_9))
        {
            level = cdmw.logging.LogLevel.DEBUG_9;
        }
        else if (strLevel.equals(LogLevel.sDEBUG))
        {
            level = cdmw.logging.LogLevel.DEBUG;
        }

        else if (strLevel.equals(LogLevel.sCDMW_DEBUG_1))
        {
            level = cdmw.logging.LogLevel.CDMW_DEBUG_1;
        }
        else if (strLevel.equals(LogLevel.sCDMW_DEBUG_2))
        {
            level = cdmw.logging.LogLevel.CDMW_DEBUG_2;
        }
        else if (strLevel.equals(LogLevel.sCDMW_DEBUG_3))
        {
            level = cdmw.logging.LogLevel.CDMW_DEBUG_3;
        }
        else if (strLevel.equals(LogLevel.sCDMW_DEBUG_4))
        {
            level = cdmw.logging.LogLevel.CDMW_DEBUG_4;
        }

        else if (strLevel.equals(LogLevel.sREQ_FOLLOWUP))
        {
            level = cdmw.logging.LogLevel.REQ_FOLLOWUP;
        }
        else if (strLevel.equals(LogLevel.sCALL_TRACE))
        {
            level = cdmw.logging.LogLevel.CALL_TRACE;
        }
        else if (strLevel.equals(LogLevel.sORB_TRACE))
        {
            level = cdmw.logging.LogLevel.ORB_TRACE;
        }

        else
        {
            throw new InvalidAlgorithmParameterException();
        }
        
        return level;
    }

}
