/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include "SystemMngt/platformlibrary/LogMngr.hpp"
#include "SystemMngt/platformlibrary/SynchronisablePreconditionTask.hpp"
#include "SystemMngt/platformlibrary/SynchronisableTaskFactory.hpp"
#include "SystemMngt/platformvaluetypes/SystemModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/SystemStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationAgentDeath_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationModeChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/ApplicationStatusChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/PropertyChange_impl.hpp"
#include "SystemMngt/platformvaluetypes/UserEvent_impl.hpp"
#include "idllib/CdmwPlatformMngtSystem.stub.hpp"
#include <sstream>
#include <iostream>
#include <fstream>
#include <limits>
#include <string>


namespace Cdmw
{
namespace PlatformMngt
{

class EventObserver;  

// ----------------------------------------------------------------------
// SynchronisableTaskTemporiser class.
// ----------------------------------------------------------------------

SynchronisablePreconditionTask::SynchronisablePreconditionTask ( 
        const int timeout,
        const std::string& managed_element,
        const std::string& status,
        const std::string& taskId,
        Sequencer* sequencer,
        TaskObserver* observer,
        SynchronisableTaskFactory* synchTaskFactory,
        size_t predecessorCount,
        CdmwPlatformMngt::Process_var process) 
:SynchronisableTask (taskId, sequencer, observer, predecessorCount)
 {
            
    //Initialising the variables
    m_event_obs=0;  
    //The precondition timeout after which the task is forcedly started 
    m_precond_timeout = timeout;
    //The managed element used in the precondition
    //at the moment only application are used
    m_precond_managed_element=managed_element;
    //The expected status
    m_precond_status=status;
    
    m_process=CdmwPlatformMngt::Process::_duplicate(process);
    
    m_application=CdmwPlatformMngt::Application::_narrow( m_process->get_parent() );
    
    m_system=CdmwPlatformMngt::System::_narrow(m_application->get_parent() );
    
    // Mutex and condition variable used to synchronise the start. 
    m_mutex=new OsSupport::Mutex();
    m_condition=new OsSupport::Condition(*m_mutex);
   
    m_observer_name="EventObserver_";
    m_observer_name.append(m_process->name());
    
    m_description="S: ";
    m_description.append( m_system->name() );
    m_description.append(" A: ");
    m_description.append( m_application->name() );
    m_description.append(" P: ");
    m_description.append( m_process->name() );
}

        
SynchronisablePreconditionTask::~SynchronisablePreconditionTask() throw() 
{
    delete m_mutex;
    delete m_condition;
}


void SynchronisablePreconditionTask::run()
throw()
{
    try
    {
        
        CdmwPlatformMngt::ApplicationStatus status = string_to_status(m_precond_status);

        //Getting the managed element application described in the precondition
        m_appl_to_wait = m_system->get_application(m_precond_managed_element.c_str()); 

        CORBA::String_var statusInfo;
        if( status != m_appl_to_wait->get_status(statusInfo.out()))
        {

            EventObserver* applObserver_impl= new EventObserver(m_mutex,m_condition,this,
                    m_precond_status);

            CdmwPlatformMngt::SupervisionObserver_var m_event_obs = applObserver_impl->_this();

            CdmwPlatformMngt::RegistrationDepth depth = CdmwPlatformMngt::OBJECT_ONLY; 

            m_appl_to_wait->register_observer_ext (m_observer_name.c_str(),depth,m_event_obs.in());
                        
            std::string msg_wait= m_description +"  Precondition: Waiting for "+m_precond_status+" of "+
            m_precond_managed_element;
            LogMngr::logMessage(INF,msg_wait.c_str());

            if( -1 != m_precond_timeout)
            {
                
                
                Cdmw::OsSupport::Condition::WaitReturn res;
                {
                    CDMW_MUTEX_GUARD( *m_mutex );
                    res = m_condition->wait(m_precond_timeout);
                    applObserver_impl->expired();
                
                //Unregister the observer
                m_appl_to_wait->unregister_observer_ext(m_observer_name.c_str());
                }
                if ( res == Cdmw::OsSupport::Condition::TIMEDOUT)
                {
                    std::string msg= m_description +"  Precondition: Timeout expired";
                    LogMngr::logMessage(INF,msg.c_str());
                }
                if( res == Cdmw::OsSupport::Condition::SIGNALLED)
                {
                    std::string msg= m_description +"  Precondition: Verified ";
                    LogMngr::logMessage(INF,msg.c_str());
                }
            }
            else
            {
                std::string msg= m_description +"  Precondition: Default timeout value in scenario file, the timer is"
                        " idle.";
                LogMngr::logMessage(INF,msg.c_str());
                {
                CDMW_MUTEX_GUARD( *m_mutex );
                m_condition->wait();
                applObserver_impl->expired();
                //Unregister the observer
                m_appl_to_wait->unregister_observer_ext(m_observer_name.c_str());
                }
            }
        }
        else
        {
            std::string msg= m_description +"  Precondition: condition already verified at the task start";
            LogMngr::logMessage(INF,msg.c_str());
        }
    
        
    }
    catch(CdmwPlatformMngt::IncompatibleStatus & e)
    {
        std::string msg= m_description +"  Precondition: Not valid status written "
        "in the scenario file.Only APPLICATION STATUS allowed...starting soon without any precondition";
        LogMngr::logMessage(ERR,msg.c_str());
    }
    catch(CdmwPlatformMngt::ApplicationNotFound & e)
    {
        std::string msg= m_description +"  Precondition: Application Not Found...starting soon without any precondition ";
        LogMngr::logMessage(ERR,msg.c_str());
    }
    catch(...)
    {
        std::string msg= m_description +" Unable to start the Task ";
        LogMngr::logMessage(ERR,msg.c_str()); 
    }
    bool ret = execute();

    if (!ret)
        m_sequencer->suspend();
        
        end();
}

void SynchronisablePreconditionTask::startTask() throw()
{
    if (m_nbPredecessorsToWait > 1)
    {
        --m_nbPredecessorsToWait;
        LogMngr::logMessage(INF,"The Task is waiting for the others predecessors end");
    }
    else
    {
        start();
    }
}       


CdmwPlatformMngt::ApplicationStatus SynchronisablePreconditionTask::string_to_status(std::string& s_string)
throw(CdmwPlatformMngt::IncompatibleStatus)
{

    CdmwPlatformMngt::ApplicationStatus result;
    bool check=false;

    if (0== strcmp("APPLICATION_INITIAL",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_INITIAL;
        check=true;
    }

    if (0== strcmp("APPLICATION_INITIALISING",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_INITIALISING;
        check=true;
    }

    if (0== strcmp("APPLICATION_INITIALISATION_INCOMPLETE",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_INITIALISATION_INCOMPLETE;
        check=true;
    }

    if (0== strcmp("APPLICATION_INITIALISED",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_INITIALISED;
        check=true;
    }

    if (0== strcmp("APPLICATION_RUN_REQUEST",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_RUN_REQUEST;
        check=true;
    }

    if (0== strcmp("APPLICATION_RUNNING",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_RUNNING;
        check=true;
    }

    if (0== strcmp("APPLICATION_STOPPING",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_STOPPING;
        check=true;
    }


    if (0== strcmp("APPLICATION_STOPPED",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_STOPPED;
        check=true;
    }

    if (0== strcmp("APPLICATION_PARTIAL_RUNNING",s_string.c_str()))
    {
        result= CdmwPlatformMngt::APPLICATION_PARTIAL_RUNNING;
        check=true;
    }

    if (check) 
    {
        return result;
    }
    else 
    {
        throw CdmwPlatformMngt::IncompatibleStatus();
    }
}



// ----------------------------------------------------------------------
// Event Observer class.
// ----------------------------------------------------------------------


EventObserver::EventObserver (
        OsSupport::Mutex* mutex,
        OsSupport::Condition* condition,
        Cdmw::PlatformMngt::SynchronisablePreconditionTask* task,
        std::string& status)
{
    m_mutex= mutex;
    m_condition= condition;
    m_task = task;
    m_status=status;
    m_expired=false;
};


void EventObserver::expired()
{
     m_expired=true;
}


//the notify function
void EventObserver::notify( CdmwPlatformMngt::Event* an_event )
throw( CORBA::SystemException )
{
    try
    {
        CdmwPlatformMngt::ApplicationStatusChange* app_chg = 
            dynamic_cast<CdmwPlatformMngt::ApplicationStatusChange*> (an_event);

            CdmwPlatformMngt::ApplicationStatus status_enum = m_task->string_to_status(m_status);
            if (app_chg != 0 
                    && app_chg->application_status() == status_enum 
                    && !m_expired) 
            {
                //Printing the message  
                CORBA::String_var message = an_event->to_string();
                std::cout << message.in() << std::endl;
                {
                    CDMW_MUTEX_GUARD( *m_mutex );
                    // awake the thread, if it is sleeping
                    m_condition->broadcast();
                }
            }
    }
    catch (...)
    {
        std::cerr << "Event observer exception " <<__FILE__<<__LINE__<<std::endl;
    }

}


} // End namespace PlatformMngt
} // End namespace Cdmw

