/* ===================================================================== */
/*
 * This file is part of CARDAMOM (R) which is jointly developed by THALES
 * and SELEX Sistemi Integrati. It is derivative work based on PERCO 
 * Copyright (C) THALES 2000-2003. All rights reserved.
 * Copyright (C) THALES 2004-2005. All rights reserved.
 * Copyright (C) SELEX SISTEMI INTEGRATI and THALES 2006-2007. 
 * All rights reserved.
 * 
 * CARDAMOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * CARDAMOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with CARDAMOM; see the file COPYING. If not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/* ===================================================================== */


#include <SystemMngt/idllib/CdmwPlatformMngtApplicationAgent.stub.hpp>
#include <SystemMngt/idllib/CdmwPlatformMngtProcessCallback.stub.hpp>
#include <SystemMngt/platformlibrary/EventHeaderFactory.hpp>

#include "tracecollector/LogCollector_impl.hpp"
#include "tracecollector/LogCircularBuffer.hpp"
#include "tracecollector/BufferSender.hpp"
#include "tracecollector/LogCollectorMacroDefs.hpp"
#include <Foundation/common/Assert.hpp>

namespace Cdmw
{

namespace Trace
{

::CORBA::Boolean LogCollector_impl::add_log (const CdmwLogging::StructuredLog& log)
  throw (::CORBA::SystemException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");

    try
    {
       // Forward message to platform daemon if necessary
       if(log.header.level==CdmwLogging::FATAL ||
          log.header.level==CdmwLogging::ERROR)
       {
          CORBA::String_var app_name = CORBA::string_dup(log.ctx.application_name.in());

          CdmwPlatformMngt::ApplicationAgent_var app_agent = 
             m_daemon->search_application_agent(app_name.in());

          if (!CORBA::is_nil(app_agent.in()))
          {
             CdmwPlatformMngt::ProcessMessageBroker_var msg_broker = 
                   app_agent->get_process_message_broker();
             
             if ( ! CORBA::is_nil(msg_broker.in()))
             {
                CdmwPlatformMngtBase::TimeStamp time_stamp;
                time_stamp.second = log.header.time_stamping.seconds;
                time_stamp.microsecond = log.header.time_stamping.microseconds;
             
                CdmwPlatformMngtBase::EventHeader_var event_header;
                switch(log.header.level)
                {
                   case CdmwLogging::FATAL:
                   {
                      event_header =
                         Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                            time_stamp, 
                            CdmwPlatformMngtBase::FTL);
                      break;
                   }
                   case CdmwLogging::ERROR:
                   {
                      event_header =
                         Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                            time_stamp, 
                            CdmwPlatformMngtBase::ERR);
                      break;
                   }
                   default:
                   {
                      // Should never happen
                      event_header =
                         Cdmw::PlatformMngt::EventHeaderFactory::createHeader(
                            time_stamp, 
                            CdmwPlatformMngtBase::FTL);
                      break;
                   }
                }
                // TODO: Define how to use process_name and component_name in message:
                //       Both in issuer ? add component_name in message ?
                //       Temporarly solution: issuer = process_name/component_name
                std::stringstream stream;
                stream << log.ctx.process_name.in();
                if (log.header.component_name.in() != 0x0 && strlen(log.header.component_name.in())>0)
                   stream << "/" << log.header.component_name.in();
                msg_broker->notify_message(event_header.in(),stream.str().c_str(),log.message.in());
             }
          }
       }
    }
    catch(const CORBA::SystemException& se )
    {
       // after a call to m_daemon.search_application_agent
       // or after a call to app_agent->get_process_message_broker
       // or after a call to msg_broker->notify_message
       // Platform daemon is busy, do nothing
    }

    bool status = m_log_circular_buffer->writeLog(log);
    return status;
}

void LogCollector_impl::register_trace_collector (CdmwTrace::LocalTraceCollector_ptr ref_trace_collector)
  throw (::CORBA::SystemException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_buffer_sender->register_trace_collector(ref_trace_collector);
}

void LogCollector_impl::unregister_trace_collector ()
  throw (::CORBA::SystemException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_buffer_sender->unregister_trace_collector();
}

PortableServer::POA_ptr LogCollector_impl::_default_POA()
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    return PortableServer::POA::_duplicate(m_poa);
}

void LogCollector_impl::init(size_t period, 
                             size_t circular_buffer_size,
                             size_t max_logs_to_send,
                             size_t chunk_size,
                             PortableServer::POA_ptr poa,
                             CdmwPlatformMngt::Daemon_ptr daemon)
  throw (OutOfMemoryException,
         BadParameterException)
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    m_poa = PortableServer::POA::_duplicate(poa);
    m_daemon = CdmwPlatformMngt::Daemon::_duplicate(daemon);

    try {
      m_log_circular_buffer = new LogCircularBuffer(circular_buffer_size, period,
                                                    max_logs_to_send, chunk_size,
                                                    m_buffer_sender);
    } catch (OutOfMemoryException& ofm) {
      CDMW_ERROR_LOG_COLLECTOR("No enough memory to create the circular buffer of the Log Collector" << ofm);
      throw ofm;
    } catch (BadParameterException bpe) {
      CDMW_ERROR_LOG_COLLECTOR("A bad parameter was set to create the circular buffer of the Log Collecctor" << bpe);
      throw bpe;
    }
}

LogCollector_impl::LogCollector_impl()
  throw ()
{
    CDMW_DEBUG_LOG_COLLECTOR("");
}

LogCollector_impl::~LogCollector_impl()
  throw ()
{
    CDMW_DEBUG_LOG_COLLECTOR("");
    delete m_log_circular_buffer;
    m_log_circular_buffer = 0;
}

} // End namespace Trace
} // End namespace Cdmw
